#!/usr/bin/env lua5.3

require 'luarocks.loader'
local lustache = require 'lustache'

local version = arg[1]
local release = tonumber(arg[2])


if not version then
	print("Version required")
	os.exit(1)
end

if not release then
	print("Rockspec Release required")
	os.exit(1)
end


local tmpl = [=[
package = "daemonparts"

version = "{{ version }}-{{ release }}"

description = {
	summary = "Unix service boilerplate",
	detailed = [[
		Daemonparts provides common - or 'boilerplate' - components
		for services that run on unix-like platforms. Modules include
		routines for locating configuration files, daemonization,
		forking child processes, logging, and recycling the entire
		service.
	]],
	
	license = "MIT/X11",
	homepage = "https://zadzmo.org/code/daemonparts"
}

source = {
	url = "https://zadzmo.org/code/daemonparts/downloads/daemonparts-{{ version }}.tar.gz"
}

dependencies = {
	"lua >= 5.3, < 5.4",
	"lunix",
	"cqueues"
}

build = {
	type = "builtin",
	modules = {
		["daemonparts.config"] = "daemonparts/config.lua",
		["daemonparts.daemonize"] = "daemonparts/daemonize.lua",
		["daemonparts.output"] = "daemonparts/output.lua",
		["daemonparts.output.file"] = "daemonparts/output/file.lua",
		["daemonparts.output.descriptor"] = "daemonparts/output/descriptor.lua",
		["daemonparts.output.table"] = "daemonparts/output/table.lua",
		["daemonparts.process"] = "daemonparts/process.lua",
		["daemonparts.recycle"] = "daemonparts/recycle.lua"
	},
	copy_directories = {
		'doc'
	}
}
]=]

local vars = {
	version = version,
	release = release
}

print(lustache:render(tmpl, vars))
