Daemonparts
===========

Daemonparts is a collection of Lua routines commonly used in unix-like
services, better known as 'daemons.' While conceptually simple, most of
these are tedious to implement, or tricky to get right due to corner
cases. Daemonparts attempts to make one well tested library for creating
a daemon quickly and easily: I've copied most of this code enough times,
in many subtle variations, that I broke it out into it's own library.

Modules include:

 - Daemonize: detach the program into the background, becoming a service
 - Config: locate and load a configuration file; provides simple access to the rest of the program
 - Output: Simple logging system - defaults to the terminal, can switch to log files, filterable severity levels
 - Process: Reliably fork worker processes and execute external programs
 - Recycle: Brute force reload the daemon to pick up configuration changes and avoid resource leaks

All modules are independant and can be used without forcing the
inclusion of any other module.

All modules are agnostic to any framework; the service is free to use
any main loop or flow control desired by the programmer.

[A detailed Ldoc reference manual is provided online.](/code/daemonparts/doc/)


Daemonize
---------

To fork into the background, and thus become a daemon, simply call
daemonize.detach:

	local daemonize = require 'daemonparts.daemonize'
	daemonize.detach()

This will fork into the background, close stdin/stdout/stderr, become
a session leader, and switch the working directory to '/' - thus
completely disowning the controlling session and terminal.

If a pidfile is desired, a path to the pidfile can be passed to
detach:

	daemonize.detach("/var/run/service.pid")

Also included is a heartbeat service, where the parent process won't
terminate - thus keeping the daemon in the foreground - until it has
fully started up and is ready to handle requests. This provides
immediate feedback to the sysadmin or user that something is not
properly configured. To use, simply set the second argument of daemonize
to boolean to true - and then call either daemonize.ready() or .error()
after completing startup:

	daemonize.detach( pid_file, true )

	local res, err = pcall( start_function )
	if not res then
		daemonize.error(err)
	end

	daemonize.ready()


[Full Reference Documentation](/code/daemonparts/doc/modules/daemonparts.daemonize.html)


Config
------

XXX: FIXME
[Full Reference Documentation](/code/daemonparts/doc/modules/daemonparts.config.html)


Output
------

XXX: FIXME
[Full Reference Documentation](/code/daemonparts/doc/modules/daemonparts.output.html)


Process
-------

XXX: FIXME
[Full Reference Documentation](/code/daemonparts/doc/modules/daemonparts.process.html)


Recycle
-------

XXX: FIXME
[Full Reference Documentation](/code/daemonparts/doc/modules/daemonparts.recycle.html)



Requirements
------------

Daemonparts is built entirely on top of
[Lunix](https://25thandclement.com/~william/projects/lunix.html),
also available in LuaRocks.


Installation
------------

The simplest method is via LuaRocks.

However, Daemonparts is pure Lua, and can be installed by simply
copying the individual Lua files from the distribution tarballs once
[Lunix](https://25thandclement.com/~william/projects/lunix.html) is 
installed.

You can download the distribution tarball 
[here.](https://zadzmo.org/code/daemonparts/downloads/)

SVN repository is [here.](https://svn.zadzmo.org/repo/daemonparts)


History
-------

 * v1.0 - First public release

  
License
-------

This code is provided without warrenty under the terms of the
[MIT/X11 License](http://opensource.org/licenses/MIT).
    
    
Contact Maintainer
------------------

You can reach me by email at [aaron] [at] [zadzmo.org]. 
