#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local config = require 'daemonparts.config'
local unix = require 'unix'


require 'busted.runner'()
describe("Config Module", function()

	before_each(function()
		config.reset()
	end)


	it("Finds a configuration", function()

		local cf = config.locate( "config.yml" )
		assert.is_equal( "./config.yml", cf )

	end)

	it("Fails when nothing is found", function()

		assert.is_error(function()
			print(config.locate( "nope.yml" ))
		end)

	end)

	it("Doesn't try to load directories", function()

		assert.is_error(function()
			-- will almost certainly match './%s'
			config.locate( "daemonparts" )
		end)

	end)

	it("Handles ENV override", function()

		unix.environ['DAEMON_CONFIG'] = '/tmp/foobar-2020.yml'
		local cf = config.locate('config.yml')
		assert.is_equal( '/tmp/foobar-2020.yml', cf )

	end)

	it("Handles ENV override, alternate name", function()

		config.define {
			name = 'system'
		}

		unix.environ['SYSTEM_CONFIG'] = '/tmp/foobar-2020.yml'
		local cf = config.locate('config.yml')
		assert.is_equal( '/tmp/foobar-2020.yml', cf )

	end)

	it("Handles required variables", function()

		config.define {
			required = {
				testvar1 = 'boolean',
				testvar2 = 'string'
			}
		}

		config.setup( 'config.yml' )
		assert.is_true( config.testvar1 )
		assert.is_equal( 'string', config.testvar2 )

	end)

	it("Rejects missing required variables", function()

		config.define {
			required = {
				missingvar = 'table'
			}
		}

		assert.is_error(function()
			config.setup( 'config.yml' )
		end)

	end)

	it("Rejects incorrect required variables", function()

		config.define {
			required = {
				testvar2 = 'number'
			}
		}

		assert.is_error(function()
			config.setup( 'config.yml' )
		end)

	end)

	it("Setup can find a file when not explicity provided", function()

		config.setup( 'config.yml' )
		assert.is_true( config.testvar1 )

	end)


	after_each(function()
		if unix.environ['DAEMON_CONFIG'] then
			unix.unsetenv('DAEMON_CONFIG')
		end

		if unix.environ['SYSTEM_CONFIG'] then
			unix.unsetenv('SYSTEM_CONFIG')
		end

		config.define {
			required = {}
		}
	end)

end)
