#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local unix = require 'unix'

local testfile = '/tmp/daemonparts-test-descr-' .. tostring(unix.getpid())
local descr

local output = require 'daemonparts.output'


require 'busted.runner'()
describe("Output to file descriptor Module", function()

	setup(function()
		descr = io.open( testfile, "w" )
		output.switch( 'descriptor', descr )
	end)

	before_each(function()
		unix.truncate( testfile )
		descr:seek("set")
		output.reset()
	end)

	teardown(function()
		unix.unlink(testfile)
	end)

	it("Removes only debug by default", function()

		output.filter('info')

		output.info("normal")
		output.notice("note")
		output.debug("debug")
		output.warning("warn")
		output.error("error")

		local expected_lines = {
			'normal',
			'NOTICE: note',
			'WARNING: warn',
			'ERROR: error'
		}

		local f = io.open(testfile, 'r')
		for i, expected in ipairs(expected_lines) do	-- luacheck: ignore 213
			local line = f:read("*line")
			assert.is_equal(expected, line)
		end

		f:close()

	end)


	it("Can lower the filter threshold", function()

		output.filter('debug')

		output.info("info")
		output.notice("notice this")
		output.debug("debug")
		output.error("error")


		local expected_lines = {
			'info',
			'NOTICE: notice this',
			'DEBUG: debug',
			'ERROR: error'
		}

		local f = io.open(testfile, 'r')
		for i, expected in ipairs(expected_lines) do	-- luacheck: ignore 213
			local line = f:read("*line")
			assert.is_equal(expected, line)
		end

		f:close()

	end)


	it("Can raise the filter threshold", function()

		output.filter('error')

		output.info("info")
		output.debug("debug")
		output.info("info")
		output.info("info")
		output.info("info")
		output.warning("warning")
		output.info("info")
		output.error("error")

		local expected_lines = {
			'ERROR: error'
		}

		local f = io.open(testfile, 'r')
		for i, expected in ipairs(expected_lines) do	-- luacheck: ignore 213
			local line = f:read("*line")
			assert.is_equal(expected, line)
		end

		f:close()

	end)


	it("Calling reset() doesn't overwrite", function()

		output.filter("info")

		output.info("info")
		local f = io.open(testfile, 'r')
		assert.is_match('info', f:read("*line"))
		f:close()

		output.reset()

		output.info("info2")
		local f2 = io.open(testfile, 'r')
		assert.is_equal('info', f2:read("*line"))
		assert.is_equal('info2', f2:read("*line"))
		f2:close()

	end)


	it("Rejects unknown filter values", function()

		-- sane - no error
		output.filter('debug')
		output.filter('notice')
		output.filter('info')
		output.filter('error')

		-- not sane - anything else
		assert.is_error(function()
			output.filter()
		end)

		assert.is_error(function()
			output.filter(353)
		end)

		assert.is_error(function()
			output.filter(function() end)
		end)

		assert.is_error(function()
			output.filter({})
		end)

		assert.is_error(function()
			output.filter(false)
		end)

		assert.is_error(function()
			output.filter("")
		end)

		assert.is_error(function()
			output.filter("some other string")
		end)

	end)

end)
