#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local output = require 'daemonparts.output'
local log_table = {}


require 'busted.runner'()
describe("Output to Table Module", function()

	setup(function()
		output.switch( 'table', log_table )
	end)

	before_each(function()

		while #log_table >= 1 do
			table.remove(log_table, 1)
		end

	end)


	it("Removes only debug by default", function()

		output.filter('info')

		output.info("normal")
		output.notice("note")
		output.debug("debug")
		output.warning("warn")
		output.error("error")

		local expected_lines = {
			{ level = 'info', message = 'normal' },
			{ level = 'notice', message = 'note' },
			{ level = 'warning', message = 'warn' },
			{ level = 'error', message = 'error' }
		}

		for i in ipairs(expected_lines) do
			assert.is_equal(expected_lines[i].message, log_table[i].message)
			assert.is_equal(expected_lines[i].level, log_table[i].level)
			assert.is_number(log_table[i].timestamp)
		end

	end)


	it("Can lower the filter threshold", function()

		output.filter('debug')

		output.info("info")
		output.notice("notice this")
		output.debug("debug")
		output.error("error")


		local expected_lines = {
			{ level = 'info', message = 'info' },
			{ level = 'notice', message = 'notice this' },
			{ level = 'debug', message = 'debug' } ,
			{ level = 'error', message = 'error' }
		}

		for i in ipairs(expected_lines) do
			assert.is_equal(expected_lines[i].message, log_table[i].message)
			assert.is_equal(expected_lines[i].level, log_table[i].level)
			assert.is_number(log_table[i].timestamp)
		end

	end)


	it("Can raise the filter threshold", function()

		output.filter('error')

		output.info("info")
		output.debug("debug")
		output.info("info")
		output.info("info")
		output.info("info")
		output.warning("warning")
		output.info("info")
		output.error("error")

		local expected_lines = {
			{ level = 'error', message = 'error' }
		}

		for i in ipairs(expected_lines) do
			assert.is_equal(expected_lines[i].message, log_table[i].message)
			assert.is_equal(expected_lines[i].level, log_table[i].level)
			assert.is_number(log_table[i].timestamp)
		end

	end)


	it("Calling reset() doesn't error", function()
		output.reset()
	end)


	it("Rejects unknown filter values", function()

		-- sane - no error
		output.filter('debug')
		output.filter('notice')
		output.filter('info')
		output.filter('error')

		-- not sane - anything else
		assert.is_error(function()
			output.filter()
		end)

		assert.is_error(function()
			output.filter(353)
		end)

		assert.is_error(function()
			output.filter(function() end)
		end)

		assert.is_error(function()
			output.filter({})
		end)

		assert.is_error(function()
			output.filter(false)
		end)

		assert.is_error(function()
			output.filter("")
		end)

		assert.is_error(function()
			output.filter("some other string")
		end)

	end)

end)
