#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local output = require 'daemonparts.output'


require 'busted.runner'()
describe("Output Module", function()

	before_each(function()
		output.reset()
	end)

	it("Prints to stdout by default", function()

		local outs = {}
		output.print = function( ... )
			outs[ #outs + 1 ] = { ... }
		end

		output.info("normal")
		output.notice("note")
		output.debug("debug")
		output.warning("warn")
		output.error("error")

		assert.is_equal(4, #outs)
		assert.is_equal(1, #( outs[1] ))
		assert.is_equal("normal", outs[1][1])

		assert.is_equal(2, #( outs[2] ))
		assert.is_equal("NOTICE:", outs[2][1])
		assert.is_equal("note", outs[2][2])

		assert.is_equal(2, #( outs[3] ))
		assert.is_equal("WARNING:", outs[3][1])
		assert.is_equal("warn", outs[3][2])

		assert.is_equal(2, #( outs[4] ))
		assert.is_equal("ERROR:", outs[4][1])
		assert.is_equal("error", outs[4][2])

	end)


	it("Can lower the filter threshold", function()

		local outs = {}
		output.print = function( ... )
			outs[ #outs + 1 ] = { ... }
		end

		output.filter('debug')

		output.info("info")
		output.notice("notice this")
		output.debug("debug")
		output.error("error")

		assert.is_equal(4, #outs)
		assert.is_equal(1, #( outs[1] ))
		assert.is_equal("info", outs[1][1])

		assert.is_equal(2, #( outs[2] ))
		assert.is_equal("NOTICE:", outs[2][1])
		assert.is_equal("notice this", outs[2][2])

		assert.is_equal(2, #( outs[3] ))
		assert.is_equal("DEBUG:", outs[3][1])
		assert.is_equal("debug", outs[3][2])

		assert.is_equal(2, #( outs[4] ))
		assert.is_equal("ERROR:", outs[4][1])
		assert.is_equal("error", outs[4][2])

	end)


	it("Can raise the filter threshold", function()

		local outs = {}
		output.print = function( ... )
			outs[ #outs + 1 ] = { ... }
		end

		output.filter('error')

		output.info("info")
		output.info("info")
		output.info("info")
		output.debug("debug")
		output.info("info")
		output.info("info")
		output.error("error")

		assert.is_equal(1, #outs)
		assert.is_equal(2, #( outs[1] ))
		assert.is_equal("ERROR:", outs[1][1])
		assert.is_equal("error", outs[1][2])

	end)


	it("Rejects unknown filter values", function()

		-- sane - no error
		output.filter('debug')
		output.filter('notice')
		output.filter('info')
		output.filter('error')

		-- not sane - anything else
		assert.is_error(function()
			output.filter()
		end)

		assert.is_error(function()
			output.filter(353)
		end)

		assert.is_error(function()
			output.filter(function() end)
		end)

		assert.is_error(function()
			output.filter({})
		end)

		assert.is_error(function()
			output.filter(false)
		end)

		assert.is_error(function()
			output.filter("")
		end)

		assert.is_error(function()
			output.filter("some other string")
		end)

	end)


	it("Rejects unknown output modules", function()

		assert.is_error(function()
			output.switch('erufnejfnejfe')
		end)

	end)

end)
