#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local process = require 'daemonparts.process'
local proc_lib = require 'tests.lib.process'

local unix = require 'unix'


require 'busted.runner'()
describe("Recycler Module", function()

	it("Recycles", function()

		local pid = unix.getpid()

		local proc = process.exec_cmd('./tests/lib/mock_recycler.lua')

		assert.is_table(proc)
		assert.is_number(proc.pid)

		local pid_line = proc.stdout:read("*line")
		assert.is_equal(proc.pid, tonumber(pid_line))

		local number_line = proc.stdout:read("*line")

		proc.stdin:write("recycle\n")
		proc.stdin:flush()

		local pid_line2 = proc.stdout:read("*line")
		assert.is_equal(proc.pid, tonumber(pid_line2))
		assert.is_true(proc_lib.process_exists( proc.pid ))

		local number_line2 = proc.stdout:read("*line")
		assert.is_not_equal(number_line, number_line2)

		proc.stdin:write("stop\n")
		proc.stdin:flush()

		local end_line = proc.stdout:read("*line")
		assert.is_equal("ending", end_line)

		assert.is_equal(0, proc.wait())

		assert.is_equal(pid, unix.getpid())

	end)

	it("Forks then recycles", function()
		local pid = unix.getpid()

		local proc = process.exec_cmd('./tests/lib/mock_recycler.lua')

		assert.is_table(proc)
		assert.is_number(proc.pid)

		local pid_line = proc.stdout:read("*line")
		assert.is_equal(proc.pid, tonumber(pid_line))

		local number_line = proc.stdout:read("*line")

		proc.stdin:write("fork\n")
		proc.stdin:flush()

		assert.is_equal(0, proc.wait())

		local pid_line2 = proc.stdout:read("*line")
		local new_pid = tonumber(pid_line2)
		assert.is_not_equal(proc.pid, new_pid)

		assert.is_false(proc_lib.process_exists( proc.pid ))
		assert.is_true(proc_lib.process_exists( new_pid ))

		local number_line2 = proc.stdout:read("*line")
		assert.is_not_equal(number_line, number_line2)

		proc.stdin:write("stop\n")
		proc.stdin:flush()

		local end_line = proc.stdout:read("*line")
		assert.is_equal("ending", end_line)

		assert.is_equal(pid, unix.getpid())

	end)

	after_each(function()
		package.loaded['luacov.runner'].save_stats()
	end)

end)
