#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local signals = require 'daemonparts.signals'

local cqueues = require 'cqueues'
local cqsignal = require 'cqueues.signal'


require 'busted.runner'()
describe("Signal Responder Module", function()

	local cq
	local callback_hit

	setup(function()

		local cb = function()
			callback_hit = true
		end

		cq = cqueues.new()
		signals.set_callback( cb )

	end)

	before_each(function()
		callback_hit = false
		signals.start(cq)
	end)

	it("Receives Stop Signals", function()

		assert.is_false(signals.is_stopping())
		assert.is_false(signals.is_reloading())

		cq:wrap(function()
			repeat
				assert.is_false(signals.is_stopping())
				assert.is_false(signals.is_reloading())
				cqueues.sleep(1)
				cqsignal.raise( cqsignal.SIGTERM )
			until signals.is_stopping()
		end)

		for err in cq:errors() do
			if err then
				error(err)
			end
		end

		while cq:count() > 0 do cq:loop(1) end

		assert.is_true(callback_hit)
		assert.is_true(signals.is_stopping())
		assert.is_false(signals.is_reloading())

	end)


	it("Receives Stop Signals, interrupted", function()

		assert.is_false(signals.is_stopping())
		assert.is_false(signals.is_reloading())

		cq:wrap(function()
			repeat
				assert.is_false(signals.is_stopping())
				assert.is_false(signals.is_reloading())
				cqueues.sleep(1)
				cqsignal.raise( cqsignal.SIGINT )
			until signals.is_stopping()
		end)

		for err in cq:errors() do
			if err then
				error(err)
			end
		end

		while cq:count() > 0 do cq:loop(1) end

		assert.is_true(callback_hit)
		assert.is_true(signals.is_stopping())
		assert.is_false(signals.is_reloading())

	end)


	it("Receives Reload Signals", function()

		assert.is_false(signals.is_stopping())
		assert.is_false(signals.is_reloading())

		cq:wrap(function()
			repeat
				assert.is_false(signals.is_stopping())
				assert.is_false(signals.is_reloading())
				cqueues.sleep(1)
				cqsignal.raise( cqsignal.SIGHUP )
			until signals.is_stopping()
		end)

		for err in cq:errors() do
			if err then
				error(err)
			end
		end

		while cq:count() > 0 do cq:loop(1) end

		assert.is_true(callback_hit)
		assert.is_true(signals.is_stopping())
		assert.is_true(signals.is_reloading())

	end)

end)
