Config File Handling
====================

This module attempts to locate a configuration file without knowing
it's location. It is intended to be called early in the script, before
a daemon detaches. Afterwards, it provides a singleton so that other 
modules can read the configuration quickly.

The file is intended to be a YAML file, however, the search and loading
fuctions are kept seperate if you wish to use a different format.


Loading the Configuration
-------------------------

Simply call 'setup' with the name of the file you wish to locate.
Sensible defaults will find it most of the time. The YAML data will
be loaded and stored in the loaded module's table:

	local config = require 'daemonparts.config'
	config.setup( 'service.yml' )
	
	if config.variable_one then
		do_function_one()
	end
	
	if config.max_files > 1000 then
		print("That's a lot of files!")
	end
	

If you wish to specify an explicit path - say, the user passed the
config file location in a command line argument - you can push a path
to try first as a second argument:

	local config = require 'daemonparts.config'
	config.setup( 'service.yml', '/try/here/first/service.yml' )


Accessing configuration elsewhere
---------------------------------

The module includes a singleton - so any of your Lua code defined in
other files can simply 'require' and access it. For example:

	local config = require 'daemonparts.config'
	
	local module = {}

	function module.check_bar()
		print("Bar is: " .. config.bar)
	end

	return module



Using your own file format
--------------------------

Most of the magic in this module is in the function 'locate'. This
will return the path to a located file, and nothing else, allowing
your code to take over.

	local config = require 'daemonparts.config'
	local path = config.locate('sysrc.json')
	
	print(path)
	-- example output: '/usr/local/etc/sysrc.json'

Doing this breaks the singleton, as well as any sanity checking, the
module normally provides.


Reloading the configuration
-------------------------

If you need to reload your configuration (say, after a SIGHUP or
explicit reload command), call reset and the repeat the initalization:

	config.reset()
	config.setup( nil, 'service.yml' )
	
	
