Daemonization
=============

The simplest way to daemonize a Lua script using Daemonparts is to
simply call 'detach':

	local daemonize = require 'daemonparts.daemonize'
	daemonize.detach()

And that's it. Your script has now forked into the background. The
underlying step it has taken are:

 - Forked, and the parent exited
 - Closed STDIN, STDOUT, and STDERR and recrated them from /dev/null
 - Became the session leader, thus disconnecting from the controlling
   terminal
 - Switched the working directory to '/'


Heartbeat Usage
---------------

Often times, daemons fail to initalize some resource they need after
they have switched to the background - thus failing to start, but
giving the user who started the process no feedback that a failure
occured, or why.

The heartbeat feature keeps the parent process in the foreground until
the child gives an all-clear signal. This way, both a non-zero exit
status and an error message can be given directly to the user (or
init system), greatly simplifying troubleshooting.

To use the heartbeat, call daemonize() with a boolean true as it's
argument. The parent will then wait until the child calls either
ready() or error() before it exits.


	local daemonize = require 'daemonparts.daemonize'
	daemonize.detach(true)

	local success, err = pcall(startup_function)
	if success then
		daemonize.ready()
	else
		daemonize.error( err )
		os.exit(15)
	end


This pattern was common enough it was wrapped into it's own
function, go().

	local daemonize = require 'daemonparts.daemonize'
	daemonize.go( startup_function )
