Output Module
=============

This module provides a simple logging framework. By default, all output
is sent to STDOUT using print(); however other targets are available.
Filtering by severity - such as silencing debug messages - is also
possible.


Usage
-----

Simply require the module. Functions are provided which replicate
Syslog-style severity levels.

	local output = require 'daemonparts.output'
	
	output.debug( "This is debug" )
	output.info( "This is general information" )
	output.notice( "This is something the user should look at" )
	output.warning( "Uh, we have a problem" )
	output.error( "OMG SCRIPT IS GOING TO DIE" )

To be clear, all of these functions only handle output - calling 'error'
will not cause the program to terminate, nor cause pcall() to fail.


Filtering Messages
------------------

Simply set the 'priorty':

	local output = require 'daemonparts.output'
	
	output.filter( 'notice' )
	
	output.debug( "This is debug 1" )
	output.debug( "This is debug 2" )
	output.notice( "Hello, World" )
	output.debug( "This is debug 3" )
	output.debug( "This is debug 4" )
	
	
This program will only output one line - Hello, World. The filter
level can be raised or lowered at any time, and defaults to 'info'.


Using Different Output
----------------------

Daemons rarely log to standard out. Calling 'switch' allows messages
to be routed to other places. The following example redirects output
to the file '/var/log/daemon.log':
	
	local output = require 'daemonparts.output'
	
	output.switch( 'file', '/var/log/daemon.log' )
	output.info( "Daemon started" )

Current output modules are file, table, file descriptor - ie, a socket.
See the full 
[Ldoc Documentation](/code/daemonparts/doc/modules/daemonparts.output.html) for details.


Resetting to defaults
---------------------

Output can be sent back to standard out at any time by calling reset().
This will also reset the filter priority.

	local output = require 'daemonparts.output'
	
	output.switch( 'file', '/var/log/daemon.log' )
	output.reset()
	output.info( "This message goes to standard out" )

The output module internally contains a singleton, so if output is
redirected anywhere, the entire program's output will be redirected.

