Process Module
==============

This module provides a way to create and control child processes.
Children can be either Lua code specified in your program - or
they can safely call exec() and spawn a completely different external
program.


Forking
-------

Calling process.fork_fcn() will create a child:

	local process = require 'daemonparts.process'
	
	local child = process.fork_fcn (
		function()
			io.stdout:write( "stdout\n" )
			io.stdout:flush()
		end
	)
	
	print( child.pid ) -- should print a varying number
	print( child.stdout:read("*\n") ) -- should print 'stdout'
	print( child:wait() ) -- should print '0'


This forks the child process, and calls the function provided by
'entry' in the child. Due to the semantics of the fork system call,
all upvalues within the called function are availble, with one caveat:
Any changes made will NOT be visible to the parent process.

However, the parent has access to the child's standard io, via the
fields '.stdout', '.stderr', and '.stdin'. Another common means is to 
open a pipe or socket, and have communication take place over that file 
descriptor. See LuaPosix, Lunix, or cqueues projects for ways to do so.

When the entry function of the child finishes, it terminates,
automatically, leaving an exit code of 0. If error() is called and not
caught, the child terminates automatically as well, leaving a nonzero
exit status.

The parent can wait for the child to terminate, and also acquire it's
exit status, with the provided wait function.
	
The function fork() takes a table as it's argument, instead of a
function; allowing various options to be set. See 
[Ldoc Documentation](/code/daemonparts/doc/modules/daemonparts.process.html) for
details.


Executing other programs
------------------------

In addtion to fork_fcn() and fork(), there is also exec_cmd() and
exec(). 

	local process = require 'daemonparts.process'
	
	local child = process.exec_cmd( "/bin/ls", "/" )
	
	print( child.pid ) -- should print a varying number
	
	for line in child.stdout:lines() do
		print(line)	-- prints a listing of the root directory
	end
	
	print( child:wait() ) -- should print '0'
	
	
exec_cmd() works much like fork_fcn(); likewise, exec() takes a table
argument containing most of the same options as fork().
