Signal Handler
==============

This module handles the most common signal conventions in unix daemons:
Graceful stop on SIGINT; configuration reload on SIGHUP. SIGTERM is
also wired to graceful stop, for usage when the daemon isn't detached
and the user presses CTRL-C on their terminal.

This module is built on cqueues and thus requires it.

Basic usage is to start the module, and then check it's state within
your daemon's main loop.

	local cqueues = require 'cqueues'
	local signals = require 'daemonparts.signals'

	local cq = cqueues.new()
	signals.start( cq )

	repeat
		-- do stuff
	until signals.is_stopping()


Reloading
---------

Many daemons react to SIGHUP by reloading their configuration.
Sending a SIGHUP will cause is_reloading() to return true, as well as
is_stopping() returning true. After the main loop exits, checking
is_reloading can tell you if it was a hangup signal and thus it's
time to start up again.

	function go()
		signals.start( cq )

		repeat
			-- do stuff
		until signals.is_stopping()
	end

	repeat

		load_config()
		go()

	until (not signals.is_reloading())

Note that, signals.start() starts a cqueues thread, that terminates
when a signal is received. Your program will need to call
signals.start() again after a SIGHUP if it intends to continue running.


Callbacks
---------

The callback handler runs as a background cqueues thread, intended
for other threads to check for state changes at their convience.
However, sometimes, the program needs to react immediately. A callback
can be set for this case:

	function stop_notification()
		print("Shutting down!")
	end

	signals.set_callback( stop_notification )
	signals.start( cq )

Callbacks are given no arguments and any return values are ignored.
As many as needed can be set; all will be called in the order in which
they were provided.
